/********************************************************************
 FileName:     	eeprom.c
 Dependencies:	See INCLUDES section
 Processor:		24F
 Hardware:		
 Complier:  
 Company:		UB
********************************************************************/

/** INCLUDES  ******************************************************/
#include "eeprom.h"

unsigned int punteroWREEPROM_INI;
unsigned int punteroWREEPROM_FIN;
unsigned int punteroRDEEPROM;
/** FUNCIONES ******************************************************/


/********************************************************************
 * Function:       	bool SaveEventoEEPROM(void)
 *
 * PreCondition:    None
 *
 * Input:           int day, int month, int year, int hour, int minute, int second
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM
 *
 * Note:            None
 *******************************************************************/

BOOL SaveEventoEEPROM(MyByte Dato0, MyByte Dato1, MyByte Dato2, MyByte Dato3, MyByte Dato4, MyByte Dato5)
{ 
	MyByte  ControlByte;		//Control Byte
	unsigned char  LowAdd, HighAdd;	//Low and High Address byte
	MyByte Data[6];
	unsigned int PunteroAux;
	BOOL retorno=1;

	Data[0]=Dato0;
	Data[1]=Dato1;
	Data[2]=Dato2;
	Data[3]=Dato3;
	Data[4]=Dato4;
	Data[5]=Dato5;
	//Cada memoria eeprom tiene 128.000 (64000 Bloque0/1 10666 grabaciones) posiciones cada pulsacin son 6 bytes  
	//0		0		5		INI BLOQUE 0 MODULO 0	
	//10665	63990	63995	FIN BLOQUE 0 MODULO 0	
	//		63996	64000	LIBRE	
	//10666	0		5		INI BLOQUE 1 MODULO 0	
	//21331	63990	63995	FIN BLOQUE 1 MODULO 0	
	//		63996	64000	LIBRE	
	//21332	0		5		INI BLOQUE 0 MODULO 1	
	//31997	63990	63995	FIN BLOQUE 0 MODULO 1	
	//		63996	64000	LIBRE	
	//31998	0		5		INI BLOQUE 1 MODULO 1	
	//42663	63990	63995	FIN BLOQUE 1 MODULO 1	
	//		63996	64000	LIBRE	
	//42664	0		5		INI BLOQUE 0 MODULO 2						
	//53329	63990	63995	FIN BLOQUE 0 MODULO 2						
	//		63996	64000			LIBRE						
	//53330	0		5		INI BLOQUE 1 MODULO 2						
	//63995	63990	63995	FIN BLOQUE 1 MODULO 2						
	//		63996	64000	LIBRE						
					
	// byte PowerOffOk=0X00 (HighAdd==0X00 LowAdd==0x08)
	//bit 3 --> Id moduloH
	//bit 2 --> Id moduloH
	//bit 1 --> Bloque 0--> 0-512KB 1--> 513-->1024KB
	//bit 0 --> R/!W 1-->Read 0 Write
	int i;
	if(punteroWREEPROM_FIN<21332)
	{
		//Primer modulo eeprom
		if(punteroWREEPROM_FIN<=10665)
		{
			//Primer bloque eeprom
			ControlByte.byte=K_Control_M0_B0;
			PunteroAux=punteroWREEPROM_FIN*6;
		}
		else
		{
			//Segundo bloque eeprom
			ControlByte.byte=K_Control_M0_B1;
			PunteroAux=(punteroWREEPROM_FIN-10666)*6;
		}
	}
	if(punteroWREEPROM_FIN>=21332 && punteroWREEPROM_FIN<42664)
	{
		//Segundo modulo eeprom
		if(punteroWREEPROM_FIN<=31997)
		{
			//Primer bloque eeprom
			ControlByte.byte=K_Control_M1_B0;
			PunteroAux=(punteroWREEPROM_FIN-21332)*6;
		}
		else
		{
			//Segundo bloque eeprom
			ControlByte.byte=K_Control_M1_B1;
			PunteroAux=(punteroWREEPROM_FIN-31998)*6;
		}
	}
	if(punteroWREEPROM_FIN>=42664)
	{
		//tercer modulo modulo eeprom
		if(punteroWREEPROM_FIN<=53329)
		{
			//Primer bloque eeprom
			ControlByte.byte=K_Control_M2_B0;
			PunteroAux=(punteroWREEPROM_FIN-42664)*6;
		}
		else
		{
			//Segundo bloque eeprom
			ControlByte.byte=K_Control_M2_B1;
			PunteroAux=(punteroWREEPROM_FIN-53330)*6;
		}
	}
	for(i=0;i<6;i++)
	{
		HighAdd=(PunteroAux&0xFF00)>>8;
		LowAdd=PunteroAux&0x00FF;
		retorno=SaveByteEEPROM(ControlByte,HighAdd,LowAdd,Data[i].byte);
		PunteroAux++;
	}
	punteroWREEPROM_FIN++;

	if(punteroWREEPROM_INI!=0)
	{
		punteroWREEPROM_INI++;
		if(punteroWREEPROM_INI>63995)
		{
			punteroWREEPROM_INI=0;
		}
	}

	if(punteroWREEPROM_FIN>63995)
	{
		punteroWREEPROM_FIN=0;
		punteroWREEPROM_INI=1;
	}

	return retorno;
}

/********************************************************************
 * Function:        SavePunterosEEPROM()
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM Punteros
 *
 * Note:            None
 *******************************************************************/

BOOL SavePunterosEEPROM()
{ 

	BOOL retorno=1;
	MyByte  ControlByte;		//Control Byte
	unsigned char  LowAdd, HighAdd;	//Low and High Address byte
	unsigned char Data[4];
	ControlByte.byte=K_Control_M0_B0;

	Data[0]=punteroWREEPROM_INI&0x00FF;
	Data[1]=punteroWREEPROM_INI>>8;
	Data[2]=punteroWREEPROM_FIN&0x00FF;
	Data[3]=punteroWREEPROM_FIN>>8;

	//HDPageWriteI2C(ControlByte.byte, HighAdd, LowAdd, Data);

	HighAdd=ADDR_PUNT_INI_L_EEPROM_H;
	LowAdd=ADDR_PUNT_INI_L_EEPROM_L;
	if(!SaveByteEEPROM(ControlByte, HighAdd, LowAdd, Data[0]))		retorno=0;
	HighAdd=ADDR_PUNT_INI_H_EEPROM_H;
	LowAdd=ADDR_PUNT_INI_H_EEPROM_L;
	if (!SaveByteEEPROM(ControlByte, HighAdd,LowAdd,  Data[1]))		retorno=0;
	HighAdd=ADDR_PUNT_FIN_L_EEPROM_H;
	LowAdd=ADDR_PUNT_FIN_L_EEPROM_L;
	if(!SaveByteEEPROM(ControlByte, HighAdd, LowAdd,  Data[2]))		retorno=0;
	HighAdd=ADDR_PUNT_FIN_H_EEPROM_H;
	LowAdd=ADDR_PUNT_FIN_H_EEPROM_L;
	if(!SaveByteEEPROM(ControlByte, HighAdd, LowAdd,  Data[3]))		retorno=0;

	return retorno;
}

BOOL SavePunterosEEPROMsinComp()
{ 

	BOOL retorno=1;
	MyByte  ControlByte;		//Control Byte
	unsigned char  LowAdd, HighAdd;	//Low and High Address byte
	unsigned char Data[4];
	ControlByte.byte=K_Control_M0_B0;

	Data[0]=punteroWREEPROM_INI&0x00FF;
	Data[1]=punteroWREEPROM_INI>>8;
	Data[2]=punteroWREEPROM_FIN&0x00FF;
	Data[3]=punteroWREEPROM_FIN>>8;

	//HDPageWriteI2C(ControlByte.byte, HighAdd, LowAdd, Data);

	HighAdd=ADDR_PUNT_INI_L_EEPROM_H;
	LowAdd=ADDR_PUNT_INI_L_EEPROM_L;
	if(!SaveByteEEPROMsinComp(ControlByte, HighAdd, LowAdd, Data[0]))		retorno=0;
	HighAdd=ADDR_PUNT_INI_H_EEPROM_H;
	LowAdd=ADDR_PUNT_INI_H_EEPROM_L;
	if (!SaveByteEEPROMsinComp(ControlByte, HighAdd,LowAdd,  Data[1]))		retorno=0;
	HighAdd=ADDR_PUNT_FIN_L_EEPROM_H;
	LowAdd=ADDR_PUNT_FIN_L_EEPROM_L;
	if(!SaveByteEEPROMsinComp(ControlByte, HighAdd, LowAdd,  Data[2]))		retorno=0;
	HighAdd=ADDR_PUNT_FIN_H_EEPROM_H;
	LowAdd=ADDR_PUNT_FIN_H_EEPROM_L;
	if(!SaveByteEEPROMsinComp(ControlByte, HighAdd, LowAdd,  Data[3]))		retorno=0;

	return retorno;
}
/********************************************************************
 * Function:        SaveAveriasEEPROM(MyByte Averias1, MyByte Averias2)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM Punteros
 *
 * Note:            None
 *******************************************************************/

BOOL SaveAveriasEEPROM(MyByte Averias1, MyByte Averias2)
{ 

	BOOL retorno=1;
	MyByte  ControlByte;		//Control Byte
	unsigned char  LowAdd, HighAdd;	//Low and High Address byte
	unsigned char Data[2];
	ControlByte.byte=K_Control_M0_B1;

	Data[0]=Averias1.byte&0x00FF;
	Data[1]=Averias2.byte&0x00FF;

	HighAdd=ADDR_AVERIA1_EEPROM_H;
	LowAdd=ADDR_AVERIA1_EEPROM_L;
	if(!SaveByteEEPROM(ControlByte, HighAdd,LowAdd,  Data[0]))		retorno=0;
	HighAdd=ADDR_AVERIA2_EEPROM_H;
	LowAdd=ADDR_AVERIA2_EEPROM_L;
	if(!SaveByteEEPROM(ControlByte, HighAdd,LowAdd,  Data[1]))		retorno=0;
	return retorno;
}

/********************************************************************
 * Function:        SaveAveriasEEPROM(MyByte Averias1, MyByte Averias2)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM Punteros
 *
 * Note:            None
 *******************************************************************/

BOOL SaveByteControlEEPROM()
{ 

	BOOL retorno=1;
	MyByte  ControlByte;		//Control Byte
	unsigned char  LowAdd, HighAdd;	//Low and High Address byte
	unsigned char Data;
	ControlByte.byte=K_Control_M0_B1;
	Data=0xAA;
	HighAdd=ADDR_CONTROL_EEPROM_H;
	LowAdd=ADDR_CONTROL_EEPROM_L;
	if(!SaveByteEEPROMsinComp(ControlByte, HighAdd,LowAdd,  Data))		retorno=0;
	return retorno;
}
/********************************************************************
 * Function:        BOOL SaveParametrosEEPROM_RTC(MyByte Parametros[100])
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM Punteros
 *
 * Note:            None
 *******************************************************************/
BOOL SaveParametrosEEPROM_RTC(MyByte Parametros[100])
{ 
	BOOL retorno=1;
	unsigned char  Add;	//Address byte
	int i;
	for (i=0;i<99;i++)
	{
		Add=0x00+i;
		if(!SaveByteEEPROM_RTC(Add,Parametros[i].byte))									retorno=0;
	}
	//GUARDAMOS LOS BYTES DE CONTROL PARA SABER QUE LOS PARAMETROS HAN SIDO CONFIGURADOS
	Add=ADDR_EEPROM_PARAM_CONT_1;
	if(!SaveByteEEPROM_RTC(Add,0x55))		retorno=0;
	Add=ADDR_EEPROM_PARAM_CONT_2;
	if(!SaveByteEEPROM_RTC(Add,0xAA))		retorno=0;

	return retorno;
}
/********************************************************************
 * Function:        BOOL SaveByteEEPROM(MyByte ControlByte, unsigned char HighAddr,unsigned char LowAddr, unsigned char Data)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM un byte realizando comprobaciones
 *
 * Note:            None
 *******************************************************************/
BOOL SaveByteEEPROM(MyByte ControlByte, unsigned char HighAddr,unsigned char LowAddr, unsigned char Data)
{ 
	BOOL retorno=1;
	unsigned char DataRD;
	ControlByte.bit0=0;
	HDByteReadI2C(ControlByte.byte, HighAddr, LowAddr, &DataRD, 1);
	if(DataRD!=Data)
	{
		HDByteWriteI2C(ControlByte.byte, HighAddr, LowAddr, Data);	
		HDByteReadI2C(ControlByte.byte, HighAddr, LowAddr, &DataRD, 1);
		if(DataRD!=Data)
		{
			retorno=0;
		}
	}
	return retorno;
}

/********************************************************************
 * Function:        BOOL SaveByteEEPROM(MyByte ControlByte, unsigned char HighAddr,unsigned char LowAddr, unsigned char Data)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM un byte realizando comprobaciones
 *
 * Note:            None
 *******************************************************************/
BOOL SaveByteEEPROMsinComp(MyByte ControlByte, unsigned char HighAddr,unsigned char LowAddr, unsigned char Data)
{ 
	BOOL retorno=1;
	unsigned char DataRD;
	ControlByte.bit0=0;
	HDByteReadI2C(ControlByte.byte, HighAddr, LowAddr, &DataRD, 1);
	if(DataRD!=Data)
	{
		HDByteWriteI2C(ControlByte.byte, HighAddr, LowAddr, Data);	
	}
	return retorno;
}
/********************************************************************
 * Function:        BOOL SaveByteEEPROM_RTC(unsigned char Addr, unsigned char Data)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de grabar en EEPROM un byte realizando comprobaciones
 *
 * Note:            None
 *******************************************************************/
BOOL SaveByteEEPROM_RTC(unsigned char Addr, unsigned char Data)
{ 
	BOOL retorno=1;
	unsigned char DataRD;
	DataRD=i2c_rtcc_eeprom_rd(Addr);
	if(DataRD!=Data)
	{
		i2c_rtcc_eeprom_wr(Addr, Data);	
    	unsigned int delay_count = 20000;        
        do
        {
        	delay_count--;
       	}while(delay_count);
		DataRD=i2c_rtcc_eeprom_rd(Addr);
		if(DataRD!=Data)
		{
			retorno=0;
		}	
	}

	return retorno;
}


/********************************************************************
 * Function:        BOOL InitParametrosEEPROM_RTC(MyByte Parametros[100])
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          boolean true-->OK false-->Err
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de inicializar los parametros a 0 y llamar a graba ren eeprom
 *
 * Note:            None
 *******************************************************************/
BOOL InitParametrosEEPROM_RTC(MyByte Parametros[100])
{ 
	BOOL retorno=1;
	unsigned int i;
	//TODO INICIALIZAR A PARAMETROS POR DEFECTO
	for(i=0;i<99;i++)
	{
		Parametros[i].byte=0;
	}
	//retorno=SaveParametrosEEPROM_RTC(Parametros);
	return retorno;
}
/********************************************************************
 * Function:        BOOL	LeerEEPROM(unsigned char *char_pointer, int longitud)
 *
 * PreCondition:    None
 *
 * Input:           Buffer a rellenar, tamao buffer
 *
 * Output:          BOOL 1 ok 0 err
 *
 * Side Effects:    None
 *
 * Overview:        Funcion encargada de LEER DE EEPROM EXTERNA
 *
 * Note:            None
 *******************************************************************/
BOOL	LeerEEPROM(unsigned char *char_pointer, int longitud)
{
	//En una trama de 255 elementos caben 246 bytes de datos, cada dato ocupa 6 bytes 41 datos completos por trama
	//Comprimimos los datos D:Dia,M:Mes,A:Ao,H:Hora,m:Minuto,S:Segundo,C:Color,P:Pulsador,R:Reserva
	//punteroRDEEPROM
	//BYTE 1
	//	D4		D3		D2		D1		D0		M3		M2		M1
	//BYTE 2
	//	M0		A6		A5		A4		A3		A2		A1		A0
	//BYTE 3
	//	H4		H3		H2		H1		H0		m5		m4		m3
	//BYTE 4
	//	m2		m1		m0		S5		S4		S3		S2		S1
	//BYTE 5
	//	S0		C1		C0		P1		P0		RA		R9		R8
	//BYTE 6
	//	R7		R6		R5		R4		R3		R2		R1		R0
	//Convertimos datos a 6 Bytes
	BOOL retorno=1;
	MyByte	ControlByte;
	unsigned char	HighAdd;
	unsigned char	LowAdd;
	unsigned char	Data;
	unsigned int   	PunteroAux;

	int i;
	BOOL DatoVacio;
	DatoVacio=0;
	for(i=0; i<41; i++)
	{
		if(punteroRDEEPROM==punteroWREEPROM_FIN)
		{
			//Se piden mas datos de los almacenados, rellenamos con 0
			Data=0;
			DatoVacio=1;
		}
		if(!DatoVacio)
		{
			if(punteroRDEEPROM<21332)
			{
				//Primer modulo eeprom
				if(punteroRDEEPROM<=10665)
				{
					//Primer bloque eeprom
					ControlByte.byte=K_Control_M0_B0;
					PunteroAux=punteroRDEEPROM*6;
				}
				else
				{
					//Segundo bloque eeprom
					ControlByte.byte=K_Control_M0_B1;
					PunteroAux=(punteroRDEEPROM-10666)*6;
				}
			}
			if(punteroRDEEPROM>=21332 && punteroRDEEPROM<42664)
			{
				//segundo modulo eeprom
				if(punteroRDEEPROM<=31997)
				{
					//Primer bloque eeprom
					ControlByte.byte=K_Control_M1_B0;
					PunteroAux=(punteroRDEEPROM-21332)*6;
				}
				else
				{
					//Segundo bloque eeprom
					ControlByte.byte=K_Control_M1_B1;
					PunteroAux=(punteroRDEEPROM-31998)*6;
				}
			}
			if(punteroRDEEPROM>=42664)
			{
				//Tercer modulo eeprom
				if(punteroWREEPROM_FIN<=53329)
				{
					//Primer bloque eeprom
					ControlByte.byte=K_Control_M2_B0;
					PunteroAux=(punteroRDEEPROM-42664)*6;
				}
				else
				{
					//Segundo bloque eeprom
					ControlByte.byte=K_Control_M2_B1;
					PunteroAux=(punteroRDEEPROM-53330)*6;
				}
			}

			int j;
			for (j=0;j<6;j++)
			{
				HighAdd=(PunteroAux&0xFF00)>>8;
				LowAdd=PunteroAux&0x00FF;
				HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);	
				*char_pointer=Data;
				char_pointer++;
				PunteroAux++;
			}
		}
		else //DatoVacio
		{
			int j;
			for (j=0;j<6;j++)
			{
				*char_pointer=Data;
				char_pointer++;
			}
		}
		punteroRDEEPROM++;
		if(punteroRDEEPROM>63995)
		{
			punteroRDEEPROM=0;
		}
	}
	return retorno;
}
/********************************************************************
 * Function:        int	EEPROMInit(MyByte Parametros[100], MyByte *Averias1, MyByte *Averias2)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          -1 Err EEPROM RTC, -2 Err EEPROM, -3 Err Puntero, 0 Parametros sin INI
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de inicializar los parametros a 0 y llamar a graba ren eeprom
 *
 * Note:            None
 *******************************************************************/
int				EEPROMInit(MyByte Parametros[100], MyByte *Averias1, MyByte *Averias2)
{
	BOOL retorno=1;
	//COJEMOS PARAMETROS DE EEPROM RTC Y AVERIAS DE EEPROM NORMAL
	unsigned char Add;
	unsigned int i;
	MyByte  ControlByte;		//Control Byte
	unsigned char  LowAdd, HighAdd;	//Low and High Address byte
	unsigned char Data;
	//MIRAMOS POSICIONES DE CONTROL DE DATOS INICIALIZADOS
	unsigned char Init1,Init2;
	MyByte AveriasAux;
	Init1=i2c_rtcc_eeprom_rd(ADDR_EEPROM_PARAM_CONT_1);	
	Init2=i2c_rtcc_eeprom_rd(ADDR_EEPROM_PARAM_CONT_2);	
	if(Init1==0x55 && Init2==0xAA)
	{
		//PARAMETROS EEPROM INICIALIZADOS
		for (i=0;i<99;i++)
		{
			Add=0x00+i;
			Parametros[i].byte=i2c_rtcc_eeprom_rd(Add);	
		}
	}
	else
	{
		//PARAMETROS EEPROM NO INICIALIZADOS
		retorno=0;
		if(!InitParametrosEEPROM_RTC(Parametros))
		{
			retorno=-1;
		}
	}

	//AVERIAS
	ControlByte.byte=K_Control_M0_B1;
	//Las averias se guardan en la eeprom extraible
	HighAdd=ADDR_AVERIA1_EEPROM_H;
	LowAdd=ADDR_AVERIA1_EEPROM_L;
	HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);
	if(Data==0xFF)
	{
		if(!SaveByteEEPROM(ControlByte, HighAdd,LowAdd, 0))
		{
			retorno=-2;
		}
	}
	AveriasAux.byte=Data;
	*Averias1 = AveriasAux;
	HighAdd=ADDR_AVERIA2_EEPROM_H;
	LowAdd=ADDR_AVERIA2_EEPROM_L;
	HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);
	if(Data==0xFF)
	{
		if(!SaveByteEEPROM(ControlByte, HighAdd,LowAdd, 0))
		{
			retorno=-2;
		}
	}
	AveriasAux.byte=Data;
	*Averias2 = AveriasAux;

	//COJEMOS VALOR DE LOS PUNTERO DE WR DESDE EEPROM
	ControlByte.byte=K_Control_M0_B0;
	HighAdd=ADDR_PUNT_INI_L_EEPROM_H;
	LowAdd=ADDR_PUNT_INI_L_EEPROM_L;
	HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);
	punteroWREEPROM_INI=0x00FF & Data;
	HighAdd=ADDR_PUNT_INI_H_EEPROM_H;
	LowAdd=ADDR_PUNT_INI_H_EEPROM_L;
	HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);
	punteroWREEPROM_INI=punteroWREEPROM_INI | ((Data&0x00FF)<<8);
	HighAdd=ADDR_PUNT_FIN_L_EEPROM_H;
	LowAdd=ADDR_PUNT_FIN_L_EEPROM_L;
	HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);
	punteroWREEPROM_FIN=0x00FF & Data;
	HighAdd=ADDR_PUNT_FIN_H_EEPROM_H;
	LowAdd=ADDR_PUNT_FIN_H_EEPROM_L;
	HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);
	punteroWREEPROM_FIN=punteroWREEPROM_FIN | ((Data&0x00FF)<<8);
	if(punteroWREEPROM_INI==0xFFFF ||punteroWREEPROM_FIN==0xFFFF)
	{
		punteroWREEPROM_INI=0;
		punteroWREEPROM_FIN=0;
		if(!SavePunterosEEPROM())
		{
			retorno=-2;
		}
		
	}

	//CONTROL DE ULTIMO GRABADO CORRECTO
	ControlByte.byte=K_Control_M0_B1;
	HighAdd=ADDR_CONTROL_EEPROM_H;
	LowAdd=ADDR_CONTROL_EEPROM_L;
	HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1);
	if(Data==0xAA)
	{
		//dato correcto reiniciamos a FF
		Data=0xFF;
		ControlByte.byte=K_Control_M0_B1;
		if(!SaveByteEEPROM(ControlByte, HighAdd,LowAdd, Data))
		{
			retorno=-2;
		}
	}
	else
	{
		retorno=-3;
		//dato incorrecto

	}
	punteroRDEEPROM=0;
	return retorno;
}

/********************************************************************
 * Function:        unsigned int 	testEEPROM(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          0 --> OK 1 -->EEPROM 1 ERR 2 -->EEPROM 2 ERR 3 -->EEPROM 3 ERR
 *
 * Side Effects:    None
 *
 * Overview:        Funcin encargada de inicializar los parametros a 0 y llamar a graba ren eeprom
 *
 * Note:            None
 *******************************************************************/
unsigned int 	testEEPROM(void)
{
	unsigned int retorno=0;
	unsigned char Data;
	MyByte		ControlByte;
	unsigned char HighAdd,LowAdd;
	HighAdd=0x00;
	LowAdd=0x00;
	ControlByte.byte=K_Control_M0_B0;
	if( HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1))
	{
		retorno=1;	//Error deteccion EEPROM 1
	}
	ControlByte.byte=K_Control_M1_B0;
	if( HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1))
	{
		retorno=2;	//Error deteccion EEPROM 2
	}
	ControlByte.byte=K_Control_M2_B0;
	if( HDByteReadI2C(ControlByte.byte, HighAdd, LowAdd, &Data, 1))
	{
		retorno=3;	//Error deteccion EEPROM 3
	}

	return retorno;
}

void setpunteroWREEPROM_INI(unsigned int val)
{
	punteroWREEPROM_INI=val;
}
void setpunteroWREEPROM_FIN(unsigned int val)
{
	punteroWREEPROM_FIN=val;
}
unsigned int 	getpunteroWREEPROM_INI(void)
{
	return 	punteroWREEPROM_INI;
}
unsigned int 	getpunteroWREEPROM_FIN(void)
{
	return 	punteroWREEPROM_FIN;
}
void 		setpunteroRDEEPROM(unsigned int val)
{
	punteroRDEEPROM=val;
}


	