/********************************************************************
 FileName:     	i2c_Func.c
 Dependencies:	See INCLUDES section
 Processor:		24F
 Hardware:		
 Complier:  
 Company:		UB
********************************************************************/

#include "p24FJ64GB004.h"
#include "i2c.h"


/*********************************************************************
* Function:        	InitI2C()
*					Inicializacin i2c	
********************************************************************/
unsigned int InitI2C(void)
{	
	//Inicializacin de i2c

	//Baud Rate.
	I2C2BRG = 0x004E; 
	//Control
	I2C2CON = 0x1200;
	I2C2RCV = 0x0000;
	I2C2TRN = 0x0000;
	I2C2CON = 0x9200;
	//return 1;

}


/*********************************************************************
* Function:        StartI2C()
*					Genera condicion de START
********************************************************************/
unsigned int StartI2C(void)
{
	I2C2CONbits.SEN = 1;		//Generar Condicion START
	while (I2C2CONbits.SEN);	//Espera condicin START
	//return(I2C2STATbits.S);	//Retornar Estado (opcional)
	//return 1;
}


/*********************************************************************
* Function:        RestartI2C()
*					Genera concicion ReStart
********************************************************************/
unsigned int RestartI2C(void)
{

	I2C2CONbits.RSEN = 1;		//Generar Restart		
	while (I2C2CONbits.RSEN);	//Espera restart	
	//return(I2C2STATbits.S);	//Retornar Estado (opcional)
	//return 1;
}


/*********************************************************************
* Function:        StopI2C()
*					Genra condicion STOP
********************************************************************/
unsigned int StopI2C(void)
{
	I2C2CONbits.PEN = 1;		//Generar Stop 
	while (I2C2CONbits.PEN);	//Espera Stop
	//return(I2C2STATbits.P);	//Retornar Estado (opcional)
	//return 1;
}


/*********************************************************************
* Function:        WriteI2C()
*					Escribir un Byte
********************************************************************/
unsigned int WriteI2C(unsigned char byte)
{
	//while (I2C2STATbits.TRSTAT);	//Espera bus libre
	I2C2TRN = byte;					//Craga Byte en buffer Transmision
	while (I2C2STATbits.TBF);		//Espera Transmision de dato
	//return 1;
}


/*********************************************************************
* Function:        IdleI2C()
*					Espera Bus Libre
********************************************************************/
unsigned int IdleI2C(void)
{
	while (I2C2STATbits.TRSTAT);		//Espera Bus Libre
	//return 1;
}

/*********************************************************************
* Function:        HDByteWriteI2C()
*
********************************************************************/
unsigned int HDByteWriteI2C(unsigned char ControlByte, unsigned char HighAdd, unsigned char LowAdd, unsigned char data)
{
	unsigned int ErrorCode;

	IdleI2C();						//Espera Bus Libre
	StartI2C();						//Condicion START 
	WriteI2C(ControlByte);			//Escribir Byte Control
	IdleI2C();

	ErrorCode = ACKStatus();		//Return <- ACK Status
	
	WriteI2C(HighAdd);
	IdleI2C();						//Escribe High Address
	WriteI2C(LowAdd);				//Escribe Low Address
	IdleI2C();

	ErrorCode = ACKStatus();		//Return ACK Status

	WriteI2C(data);					//Escribe Dato
	IdleI2C();
	StopI2C();						//Condicion STOP
	ErrorCode=EEAckPolling(ControlByte);		//Ack Polling
	return(ErrorCode);
}


/*********************************************************************
* Function:        HDByteReadI2C()
*
********************************************************************/
unsigned int HDByteReadI2C(unsigned char ControlByte, unsigned char HighAdd, unsigned char LowAdd, unsigned char *Data, unsigned char Length)
{
	unsigned int ErrorCode=0;
	IdleI2C();						
	StartI2C();						
	WriteI2C(ControlByte);		
	IdleI2C();					

	ErrorCode = ACKStatus();	

	WriteI2C(HighAdd);			
	IdleI2C();					
	WriteI2C(LowAdd);			
	IdleI2C();					

	RestartI2C();					
	WriteI2C(ControlByte | 0x01);	
	IdleI2C();						

	getsI2C(Data, Length);		
	NotAckI2C();					
	StopI2C();						
	return(ErrorCode);
}

/*********************************************************************
* Function:        HDPageWriteI2C()
*
********************************************************************/
unsigned int HDPageWriteI2C(unsigned char ControlByte, unsigned char HighAdd, unsigned char LowAdd, unsigned char *wrptr)
{
	IdleI2C();			
	StartI2C();			
	WriteI2C(ControlByte);	
	IdleI2C();				
	WriteI2C(HighAdd);	
	IdleI2C();			
	WriteI2C(LowAdd);		
	IdleI2C();			
	putstringI2C(wrptr);	
	IdleI2C();			
	StopI2C();				
	return(0);
}

/*********************************************************************
* Function:        HDSequentialReadI2C()
*
********************************************************************/
unsigned int HDSequentialReadI2C(unsigned char ControlByte, unsigned char HighAdd, unsigned char LowAdd, unsigned char *rdptr, unsigned char length)
{
	IdleI2C();					
	StartI2C();					
	WriteI2C(ControlByte);		
	IdleI2C();					
	WriteI2C(HighAdd);			
	IdleI2C();					
	WriteI2C(LowAdd);			
	IdleI2C();						
	RestartI2C();					
	WriteI2C(ControlByte | 0x01);	
	IdleI2C();					
	getsI2C(rdptr, length);		
	NotAckI2C();					
	StopI2C();						
	return(0);
}


/*********************************************************************
* Function:        ACKStatus()
*
********************************************************************/
unsigned int ACKStatus(void)
{
	return (I2C2STATbits.ACKSTAT);		//Return <- Ack Status
}


/*********************************************************************
* Function:        NotAckI2C()
*
********************************************************************/
unsigned int NotAckI2C(void)
{
	I2C2CONbits.ACKDT = 1;		
	I2C2CONbits.ACKEN = 1;
	while(I2C2CONbits.ACKEN);		//Espera ACK 
	I2C2CONbits.ACKDT = 0;			
	//return 1;
}


/*********************************************************************
* Function:        AckI2C()
*
********************************************************************/
unsigned int AckI2C(void)
{
	I2C2CONbits.ACKDT = 0;		
	I2C2CONbits.ACKEN = 1;
	while(I2C2CONbits.ACKEN);		//Espera ACK
	//return 1;
}


/*********************************************************************
* Function:       getsI2C()
*
********************************************************************/
unsigned int getsI2C(unsigned char *rdptr, unsigned char Length)
{
	while (Length --)
	{
		*rdptr++ = getI2C();		
		
		if(I2C2STATbits.BCL)		//Si colision en el BUS
		{
			return(-1);
		}

		if(Length)
		{
			AckI2C();			
		}
	}
	return(0);
}


/*********************************************************************
* Function:        getI2C()
*
********************************************************************/
unsigned int getI2C(void)
{
//	while (I2C1STATbits.TRSTAT);		
//	I2C2CONbits.RCEN = 1       ;		
	I2C2CONbits.RCEN = 1;			//Enable Master receive
	Nop();
	while(!I2C2STATbits.RBF);		//Espera Buffer Lleno
	return(I2C2RCV);				//Retorna Dato en Buffer
}


/*********************************************************************
* Function:        EEAckPolling()
*
********************************************************************/
unsigned int EEAckPolling(unsigned char control)
{
	unsigned int Cnt_ToutAckEEP=150;  //Timeout 
	IdleI2C();				
	StartI2C();				
	
	if(I2C2STATbits.BCL)
	{
		return(-1);			//Bus colision
	}

	else
	{
		if(WriteI2C(control))
		{
			return(-3);		//error return
		}

		IdleI2C();			
		if(I2C2STATbits.BCL)
		{
			return(-1);		//error return
		}

		while( (ACKStatus()) && Cnt_ToutAckEEP--)
		{
			RestartI2C();	
			if(I2C2STATbits.BCL)
			{
				return(-1);	//error return
			}

			if(WriteI2C(control))
			{
				return(-3);
			}

			IdleI2C();
		}
	}
	StopI2C();				//stop 

	if(Cnt_ToutAckEEP==0)
	{
		return(-4);			//Timeout 
	}

	if(I2C2STATbits.BCL)
	{
		return(-1);
	}
	return(0);
}


/*********************************************************************
* Function:        putstringI2C()
*
********************************************************************/
unsigned int putstringI2C(unsigned char *wrptr)
{
	unsigned char x;

	for(x = 0; x < PAGESIZE; x++)		//DATOS hasta PAGESIZE
	{	
		if(WriteI2C(*wrptr))			//Write 1 byte
		{
			return(-3);				//Write Collision
		}
		IdleI2C();					//Bus Libre
		if(I2C2STATbits.ACKSTAT)
		{
			return(-2);				//Bus responde Not ACK
		}
		wrptr++;
	}
	return(0);
}
