/********************************************************************
 FileName:     	rtcc_func.c
 Dependencies:	See INCLUDES section
 Processor:		24F
 Hardware:		
 Complier:  
 Company:		UB
********************************************************************/
#include <p24Fxxxx.h>
#include "i2c_rtcc.h"
#include "i2c.h"
//********************************************************************************************
//                         I2C RTCC DRIVERS 
//********************************************************************************************
extern unsigned char err_flg  ;   // error flag


void i2c_rtcc_wr(unsigned char rtcc_reg, unsigned char time_var)// Escribe byte a RTC
{    
	IdleI2C();						
	StartI2C()             ;  		
	WriteI2C(ADDR_RTCC_WRITE) ;  	
	IdleI2C();						
	WriteI2C(rtcc_reg)        ;  	
	IdleI2C();						
	WriteI2C(time_var)        ;  	
	IdleI2C();						
	StopI2C()           ;  			
}  


void i2c_rtcc_eeprom_wr(unsigned char rtcc_reg, unsigned char time_var)// Escribe Byte EEPROM RTC
{    
	IdleI2C();						
	StartI2C()             ;  		
	WriteI2C(ADDR_EEPROM_WRITE) ;  	
	IdleI2C();						
	WriteI2C(rtcc_reg)        ;  	
	IdleI2C();						
	WriteI2C(time_var)        ;  	 
	IdleI2C();						
	StopI2C()           ;  			
}  

unsigned char i2c_rtcc_rd(unsigned char rtcc_reg)		// Leer byte de RTC
{ 
	unsigned char rtcc_buf	;  
	IdleI2C();						
	StartI2C()             ;  
	WriteI2C(ADDR_RTCC_WRITE) ;  
	//if(err_flg)                
	//{ return rtcc_buf   ;   }  
	IdleI2C();						
	WriteI2C(rtcc_reg)        ;  
	//if(err_flg)                
	//{ return rtcc_buf   ;   }   
	IdleI2C();					
	RestartI2C()           ;  
	IdleI2C();						
	WriteI2C(ADDR_RTCC_READ)  ;  
	//if(err_flg)                
	//{ return rtcc_buf   ;   }  
	IdleI2C();						
	rtcc_buf=getI2C()       ;  
	NotAckI2C()              ;  
	StopI2C()              ;  
	return rtcc_buf    	;   
}  

unsigned char i2c_rtcc_eeprom_rd(unsigned char rtcc_reg)		// Leer byte EEPROM RTC
{ 
	unsigned char rtcc_buf	;  
	IdleI2C();					
	StartI2C()             ;  
	WriteI2C(ADDR_EEPROM_WRITE) ;  
	//if(err_flg)                
	//{ return rtcc_buf   ;   }  
	IdleI2C();						
	WriteI2C(rtcc_reg)        ;  
	//if(err_flg)                
	//{ return rtcc_buf   ;   }  
	IdleI2C();					
	RestartI2C()           ;  
	IdleI2C();						
	WriteI2C(ADDR_EEPROM_READ)  ;  
	//if(err_flg)                
	//{ return rtcc_buf   ;   }  
	IdleI2C();						
	rtcc_buf=getI2C()       ;  
	NotAckI2C()              ;  
	StopI2C()              ;  
	return rtcc_buf    	;   
}  

void ini_i2c_rtcc(void)			//Inicializa RTC
{ 
	unsigned char day=0; 				// local variable (stores the RTCC DAY register)         
	day = i2c_rtcc_rd(ADDR_DAY); 		// read day + OSCON bit   
	i2c_rtcc_wr(ADDR_DAY,day|VBATEN); 	// enable the battery back-up  
}	 


BOOL ini_i2c_time(void)					//Inicializa Variable de Fecha/hora
{
	unsigned char day=0;   				
	BOOL retorno=1;
	i2c_rtcc_wr(ADDR_CAL,0x00);         //  Fuerza registro calibracion reloj RTCC a 0
	day = i2c_rtcc_rd(ADDR_DAY);   		
	if((day&OSCON)==OSCON) ;            // if oscillator funciona
	else
	{   
	// Oscilador no Esta en Marcha
		i2c_rtcc_wr(ADDR_YEAR,0x00);        			// inicializa YEAR  register : (20)82           
		i2c_rtcc_wr(ADDR_MNTH,0x01);   	    			// inicializa MONTH register : november   
		i2c_rtcc_wr(ADDR_DATE,0x01);   	    			// inicializa DATE  register : date = 01  
		i2c_rtcc_wr(ADDR_HOUR,0x00);   	    			// inicializa HOUR  register : hour = 00  
		i2c_rtcc_wr(ADDR_MIN,0x00) ;   	    			// inicializa MIN   register : min  = 00  
		i2c_rtcc_wr(ADDR_SEC,START_32KHZ);
		i2c_rtcc_wr(ADDR_PUNT_WR_INI_H ,0x00) ;   	     //inicializa puntero
		i2c_rtcc_wr(ADDR_PUNT_WR_INI_L ,0x00) ;   	     //inicializa puntero
		i2c_rtcc_wr(ADDR_PUNT_WR_FIN_H ,0x00) ;   	     //inicializa puntero
		i2c_rtcc_wr(ADDR_PUNT_WR_FIN_L ,0x00) ;   	     //inicializa puntero
		retorno=0;
	}
	return retorno;
} 
 
