/********************************************************************
 FileName:     	uart.c
 Dependencies:	See INCLUDES section
 Processor:		24F
 Hardware:		
 Complier:  
 Company:		UB
********************************************************************/


/* _____STANDARD INCLUDES____________________________________________________ */

/* _____PROJECT INCLUDES_____________________________________________________ */
#include "uart.h"

/* _____LOCAL DEFINITIONS____________________________________________________ */
#define UART_BAUD           	19200ul

/// Internal ring buffer sizes
#define UART_TX_BUFFER_SIZE 	256
#define UART_RX_BUFFER_SIZE 	64

/* _____GLOBAL VARIABLES_____________________________________________________ */

/* _____LOCAL VARIABLES______________________________________________________ */
/// Receive ring (circular) buffer
BOOL 			trama_recivida;
unsigned int	cnt_uart_RX_buffer;
unsigned char 	uart_RX_buffer[UART_RX_BUFFER_SIZE];
unsigned char 	uart_RX_buffer_trama[UART_RX_BUFFER_SIZE];

/// Transmit ring (circular) buffer
unsigned int	cnt_uart_TX_buffer;
unsigned char 	uart_TX_buffer[UART_TX_BUFFER_SIZE];

/* _____LOCAL FUNCTION PROTOTYPES____________________________________________ */

/* _____MACROS_______________________________________________________________ */

/* _____LOCAL FUNCTIONS______________________________________________________ */
// UART Receive interupt handler
void __attribute__((__interrupt__,auto_psv)) _U1RXInterrupt(void)
{
    unsigned char data;

    // See if Receive FIFO buffer has data
    while(U1STAbits.URXDA == 1)
    {
        // See if Receive buffer has overflowed
        if(U1STAbits.OERR == 1)
        {
            // Reset flag
            U1STAbits.OERR = 0;
        }

        // Accept data only if there were no Framing or Parity Error(s)
        if((U1STAbits.FERR == 1) || (U1STAbits.PERR == 1))
        {
            // Received data had an error, discard received data
            data = U1RXREG;
            continue;
        }

        // Fetch data
        data = U1RXREG;
		if(getTimerUart1ms()==0)
		{
			cnt_uart_RX_buffer=0;	
		}
		if(cnt_uart_RX_buffer==0 && data != 0xAA)
		//if(cnt_uart_RX_buffer==0 && data != 0x61)
		{
            // Esperando primer Byte y no llega o trama recivida no 
            continue;
		}
        // Buffer received data
        setDataBufferRX(data);
		if(cnt_uart_RX_buffer>1)
		{
			cnt_uart_RX_buffer=cnt_uart_RX_buffer;
		}
		//Cargamos un timer de 50ms
		setTimerUart1ms(500);
		if(cnt_uart_RX_buffer>4 && uart_RX_buffer[2]== cnt_uart_RX_buffer) //TODO seguridad
		{
            // Trama recivida copia de buffer a trama y borrar buffer
		  	unsigned int 	i = 0;   
   			 while(i <cnt_uart_RX_buffer) 
			{
				uart_RX_buffer_trama[i]=uart_RX_buffer[i];
        		i++;         
    		}
			cnt_uart_RX_buffer=0;
			trama_recivida=1;
            continue;
		}
    }

    // Clear Receive interrupt flag
    IFS0bits.U1RXIF = 0;
}

// UART Transmit interrupt handler
void __attribute__((__interrupt__,auto_psv)) _U1TXInterrupt(void)
{
   	unsigned char data;
    // See if Transmit FIFO buffer is not full
    while(U1STAbits.UTXBF == 0)
    {
        // Fetch data to be sent from ring buffer
        if(getDataBufferTX(&data))
        {
            // Add data to Transmit FIFO buffer
            U1TXREG = data;
        }
        else
        {
            // No more data to be sent; disable Transmit interrupt
            IEC0bits.U1TXIE = 0;
            break;
        }
    }

    // Clear Transmit interrupt flag
    IFS0bits.U1TXIF = 0;
}

/* _____GLOBAL FUNCTIONS_____________________________________________________ */
void uart1_init(void)
{
	//unlock registers
	__builtin_write_OSCCONL(OSCCON & 0xBF);
	// Configure Input Functions (Table 9-1)
	// Assign U1RX To Pin RP0
	//RPINR18bits.U1RXR = 22;
	RPINR18bits.U1RXR = 23;
	// Configure Output Functions (Table 9-2)
	// Assign U1TX To Pin RP2
	//RPOR11bits.RP23R = 3;
	RPOR11bits.RP22R = 3;
	//lock registers
	__builtin_write_OSCCONL(OSCCON | 0x40);

    U1MODEBITS u1modebits = U1MODEbits;
    U1STABITS  u1stabits  = U1STAbits;

    // Initialise ring buffers
	cnt_uart_RX_buffer=0;
	cnt_uart_TX_buffer=0;

    // Set Baud Rate register TODO
    //U1BRG = UDIV_ROUND(F_CY,(UART1_BAUD*16))-1;
	U1BRG =((16000000/57600)/16) - 1;

    // Set UART1 Mode register
    u1modebits.STSEL    = 0; // One Stop bit
    u1modebits.PDSEL0   = 0;
    u1modebits.PDSEL1   = 0; // 8-bit data, no parity
    u1modebits.BRGH     = 0; // BRG generates 16 clocks per bit period (16x baud clock, Standard mode)
    u1modebits.RXINV    = 0; // UxRX Idle state is '1'
    u1modebits.ABAUD    = 0; // Baud rate measurement disabled or completed
    u1modebits.LPBACK   = 0; // Loopback mode is disabled
    u1modebits.WAKE     = 0; // No wake-up enabled
    u1modebits.UEN0     = 0;
    u1modebits.UEN1     = 0; // UxTX and UxRX pins are enabled and used; UxCTS and UxRTS/BCLKx pins controlled by PORT latches
    u1modebits.RTSMD    = 1; // UxRTS pin in Simplex mode
    u1modebits.IREN     = 0; // IrDA encoder and decoder disabled
    u1modebits.USIDL    = 0; // Continue module operation in Idle mode
    u1modebits.UARTEN   = 1; // UARTx is enabled; all UARTx pins are controlled by UARTx as defined by UEN<1:0>

    U1MODEbits = u1modebits;

    // UART1 Status and Control register
    u1stabits.ADDEN     = 0; // Address Detect mode disabled
    u1stabits.URXISEL0  = 0;
    u1stabits.URXISEL1  = 0; // Interrupt is set when any character is received and transferred from the RSR to the receive buffer. Receive buffer has one or more characters.
    u1stabits.UTXEN     = 1; // Transmit enabled, UxTX pin controlled by UARTx
    u1stabits.UTXBRK    = 0; // Sync Break transmission disabled or completed
    u1stabits.UTXINV    = 0; // UxTX Idle '1'
    u1stabits.UTXISEL0  = 0;
    u1stabits.UTXISEL1  = 1; // Interrupt when a character is transferred to the Transmit Shift Register (TSR) and as a result, the transmit buffer becomes empty

    U1STAbits = u1stabits;
                   
    // Clear UART1 Receive and Transmit interrupt flags
    IFS0bits.U1RXIF = 0; 
    IFS0bits.U1TXIF = 0; 

    // Enable UART1 Receive interrupt
    IEC0bits.U1RXIE = 1;
}

void uart_change_baud(unsigned int baud)
{
	//TODO
    U1BRG = 0;
}

void flush_uart_rx_buffer(void)
{
	cnt_uart_RX_buffer=0;
	trama_recivida=0;
	return;
}

BOOL uart_rx_buffer_empty(void)
{
	if(cnt_uart_RX_buffer==0) 	return 1;
	else 						return 0;
}
BOOL uart_rx_buffer_full(void)
{
	if(cnt_uart_RX_buffer==UART_RX_BUFFER_SIZE) 	return 1;
	else 											return 0;
}
BOOL uart_tx_buffer_empty(void)
{
	if(cnt_uart_TX_buffer==0) 	return 1;
	else 						return 0;
}
BOOL uart_tx_buffer_full(void)
{
	if(cnt_uart_TX_buffer==UART_TX_BUFFER_SIZE) 	return 1;
	else 											return 0;
}

BOOL uart_tx_finished(void)
{
    if(cnt_uart_TX_buffer!=0)
    {
        return FALSE;
    }
    return (U1STAbits.TRMT ==1);
}

unsigned int uart_tx_data(unsigned char* buff, unsigned int bytes_to_send)
{
  	unsigned int 	i = 0;   
  	unsigned int 	j = bytes_to_send-1;        
	cnt_uart_TX_buffer=bytes_to_send;
    while(i <= j) {
        //uart_TX_buffer[i] = buff[j];
		uart_TX_buffer[i]=buff[j];
		uart_TX_buffer[j]=buff[i];
        i++;         // Move forwards with the early char
		j--;
    }
    // Manually enable UART1 Transmit interrupt
    IEC0bits.U1TXIE = 1;
    IFS0bits.U1TXIF = 1;
    return 1;
}

unsigned int uart_rx_data(unsigned char* buff, unsigned int bytes_to_read)
{
  	unsigned int 	i = 0;   
    while(i < uart_RX_buffer_trama[2]) {
		buff[i]=uart_RX_buffer_trama[i];
        i++;         
    }
    return i;
}

BOOL	setDataBufferRX(unsigned char data)
{
	BOOL retorno=1;
	if(cnt_uart_RX_buffer<UART_RX_BUFFER_SIZE)
	{
		uart_RX_buffer[cnt_uart_RX_buffer]=data;
		cnt_uart_RX_buffer++;
		retorno=1;
	}
	else
	{
		retorno=0;
	}
	return retorno;
}

BOOL 	getDataBufferTX(unsigned char *data)
{
	BOOL retorno=1;
	if(cnt_uart_TX_buffer>0)
	{
		*data=uart_TX_buffer[cnt_uart_TX_buffer-1];
		cnt_uart_TX_buffer--;
		retorno=1;
	}
	else
	{
		retorno=0;
	}
	return retorno;
}

BOOL 	getTramaUartRecivida(void)
{
	return 	trama_recivida;
}

void 	setTramaUartRecivida(BOOL valor)
{
	trama_recivida=valor;
}