/********************************************************************
 FileName:     	usb_hal.c
 Dependencies:	See INCLUDES section
 Processor:		24F
 Hardware:		
 Complier:  
 Company:		UB
********************************************************************/

#ifndef _USB_HAL_H_
#define _USB_HAL_H_

#include "usb_hal_pic24.h"
 
/**********************
 Interface Routines
 **********************/

/*************************************************************************
    Function:
        void USBHALSetBusAddress( BYTE addr )
  *************************************************************************/

#define USBHALSetBusAddress OTGCORE_SetDeviceAddr

void USBHALSetBusAddress( BYTE addr );


/*************************************************************************
    Function:
        void USBHALControlUsbResistors( BYTE flags );
 *************************************************************************/

#define USBHALControlUsbResistors OTGCORE_ControlUsbResistors
void USBHALControlUsbResistors( BYTE flags );

/* USBHALControlUsbResistors flags */
#define USB_HAL_PULL_UP_D_PLUS  0x80    // Pull D+ line high
#define USB_HAL_PULL_UP_D_MINUS 0x40    // Pull D- line high
#define USB_HAL_PULL_DN_D_PLUS  0x20    // Pull D+ line low
#define USB_HAL_PULL_DN_D_MINUS 0x10    // Pull D- line low
/*
 The following are defined for convenience:
 */
#define USB_HAL_DEV_CONN_FULL_SPD       USB_HAL_PULL_UP_D_PLUS
#define USB_HAL_DEV_CONN_LOW_SPD        USB_HAL_PULL_UP_D_MINUS
#define USB_HAL_DEV_DISCONNECT          0


/*
 MCHP: Define a method to check for SE0 & a way to send a reset (SE0).
 */


/*************************************************************************
    Function:
       BOOL USBHALSessionIsValid( void )
 
 *************************************************************************/

BOOL USBHALSessionIsValid( void );


/*************************************************************************
    Function:
        USBHALControlBusPower
               
 *************************************************************************/

BOOL USBHALControlBusPower( BYTE cmd );

/* USBHALControlBusPower Commands */
#define USB_VBUS_DISCHARGE  0       // Dicharge Vbus via resistor
#define USB_VBUS_CHARGE     1       // Charge Vbus via resistor
#define USB_VBUS_POWER_ON   3       // Supply power to Vbus
#define USB_VBUS_POWER_OFF  4       // Do not supply power to Vbus
/*
 Note: All commands except USB_VBUS_POWER_ON imply that this device
 does not actively supply power to Vbus.
 */


/*************************************************************************
    Function:
        unsigned long USBHALGetLastError( void )
              
 *************************************************************************/

unsigned long USBHALGetLastError( void );

/*
 USBHALGetLastError Error Bits.
 */
#define USBHAL_PID_ERR  0x00000001  // Packet ID Error
#define USBHAL_CRC5     0x00000002  // (Host) Token CRC5 check failed
#define USBHAL_HOST_EOF 0x00000002  // (Host) EOF not reached before next SOF
#define USBHAL_CRC16    0x00000004  // Data packet CRC error
#define USBHAL_DFN8     0x00000008  // Data field size not n*8 bits
#define USBHAL_BTO_ERR  0x00000010  // Bus turn-around timeout
#define USBHAL_DMA_ERR  0x00000020  // DMA error, unable to read/write memory
#define USBHAL_BTS_ERR  0x00000080  // Bit-stuffing error
#define USBHAL_XFER_ID  0x00000100  // Unable to identify transfer EP
#define USBHAL_NO_EP    0x00000200  // Invalid endpoint number
#define USBHAL_DMA_ERR2 0x00000400  // Error starting DMA transaction


/*************************************************************************
    Function:
        void USBHALHandleBusEvent ( void )
     
 *************************************************************************/

void USBHALHandleBusEvent ( void );


/*************************************************************************
    Function:
        BOOL USBHALStallPipe( TRANSFER_FLAGS pipe )
     
 *************************************************************************/
/*
 Note: This function is implemented as a macro, calling directly into
 an internal HAL routine.
 */

#define USBHALStallPipe OTGCORE_StallPipe

BOOL USBHALStallPipe( TRANSFER_FLAGS pipe );


/******************************************************************************
    Function:
        BOOL USBHALUnstallPipe( TRANSFER_FLAGS pipe )
  *****************************************************************************/
/*
 Note: This function is implemented as a macro, calling directly into
 an internal HAL routine.
 */

#define USBHALUnstallPipe OTGCORE_UnstallPipe

BOOL USBHALUnstallPipe( TRANSFER_FLAGS pipe );


/**************************************************************************
    Function:
        USBHALGetStalledEndpoints
 
 *************************************************************************/

/*
 Note: This function is implemented as a macro, calling directly into
 a HAL routine.
 */

#define USBHALGetStalledEndpoints OTGCORE_GetStalledEndpoints

UINT16 USBHALGetStalledEndpoints ( void );


/******************************************************************************
    Function:
        BOOL USBHALFlushPipe( TRANSFER_FLAGS pipe )
 
 *****************************************************************************/

BOOL USBHALFlushPipe( TRANSFER_FLAGS pipe );


/**************************************************************************
    Function:
        USBHALTransferData
  
        
 *************************************************************************/

BOOL USBHALTransferData ( TRANSFER_FLAGS    flags,
                          void             *buffer,
                          unsigned int      size      );


/*************************************************************************
    Function:
        USBHALSetEpConfiguration

 *************************************************************************/

BOOL USBHALSetEpConfiguration ( BYTE ep_num, UINT16 max_pkt_size, UINT16 flags );

/* Flags for USBHALSetEpConfiguration */
#define USB_HAL_TRANSMIT    0x0400  // Enable EP for transmitting data
#define USB_HAL_RECEIVE     0x0800  // Enable EP for receiving data
#define USB_HAL_HANDSHAKE   0x0100  // Enable EP to give ACK/NACK (non isoch)
/* Does not work, Fix this if needed. 3/1/07 - Bud
#define USB_HAL_NO_INC      0x0010  // Use for DMA to another device FIFO
#define USB_HAL_HW_KEEPS    0x0020  // Cause HW to keep EP
*/
#define USB_HAL_ALLOW_HUB   0x8000  // (host only) Enable low-spd hub support
#define USB_HAL_NO_RETRY    0x4000  // (host only) disable auto-retry on NACK



/*************************************************************************
    Function:
        USBHALInitialize
          
 *************************************************************************/

BOOL USBHALInitialize ( unsigned long flags );

#endif  // _USB_HAL_H_
/*************************************************************************
 * EOF
 */

